/**************************************************************************//**
*   @file   ad5421.c
*   @brief  Implementation of AD5421 Driver for NIOS processor.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#define AD5421_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "ad5421.h"
#include <altera_avalon_pio_regs.h>	//PIO access macros.
#include <altera_avalon_spi.h>		//SPI functions.
#include <unistd.h>					//usleep() function.
#include <system.h>
#if ( USE_AD5421 == 1 )

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char	txBuffer[4] 	= {0,};	// Tx SPI Buffer.
unsigned char   rxBuffer[4] 	= {0,};	// Rx SPI Buffer.

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
int* 	ad5421_error			= &AD5421_Data[0].iData;	// Indicates a spi connection error
int* 	ad5421_dacValue			= &AD5421_Data[1].iData;	// Value to be written in DAC Register
int* 	ad5421_dacValueGet		= &AD5421_Data[2].iData;    // Value read from the DAC Register
int* 	ad5421_offsetValue		= &AD5421_Data[3].iData;    // Value to be written in OFFSET Register
int* 	ad5421_offsetValueGet	= &AD5421_Data[4].iData;    // Value read from the OFFSET Register
int* 	ad5421_gainValue		= &AD5421_Data[5].iData;    // Value to be written in GAIN Register
int* 	ad5421_gainValueGet		= &AD5421_Data[6].iData;    // Value read from the GAIN Register
float* 	ad5421_vloop			= &AD5421_Data[7].fData;    // Value of the VLoop - COM Voltage
float* 	calcTemp				= &AD5421_Data[8].fData;   	// Used to calculate the temperature
float* 	loopCurValue			= &AD5421_Data[9].fData;   	// Used to calculate the current in the Voltage Loop
float* 	loopCurValueOptim		= &AD5421_Data[10].fData;   // Used to calculate the current in the Voltage Loop
char* 	faultVloop12			= &AD5421_Data[11].chData;  // VLoop under 0.6V error indicator
char* 	faultVloop6				= &AD5421_Data[12].chData;	// VLoop under 0.3V error indicator
char* 	faultTemp100			= &AD5421_Data[13].chData;	// Temperature over 100C error indicator
char* 	faultTemp140			= &AD5421_Data[14].chData;	// Temperature over 140C error indicator
char* 	faultIloopUnder			= &AD5421_Data[15].chData;	// ILoop Under error indicator
char* 	faultIloopOver			= &AD5421_Data[16].chData;	// ILoop Over error indicator
char* 	faultPec				= &AD5421_Data[17].chData;	// Package error indicator
char* 	faultSpi				= &AD5421_Data[18].chData;	// SPI Connection error indicator

/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for AD5421 Board.
 *
 * @param None.
 *
 * @return SUCCESS or FAIL.
******************************************************************************/
int AD5421_Init( void )
{
	int spiData;
	int cmdCheck = (CTRL_AUTO_FAULT_RDBK |
					CTRL_SEL_ADC_INPUT |
					CTRL_ONCHIP_ADC);						// Set bits for initialization process.
	int retValue = FAIL;
	// setup SPI interface
	SetCtrl( AD5421_SPI_CFG );
	// setup AD5421 control register
	spiData = ( AD5421_CMDWRCTRL << AD5421_CMDPOSITION ); 	// Write to the control register.
	spiData += ( CTRL_AUTO_FAULT_RDBK |
				 CTRL_SEL_ADC_INPUT |
				 CTRL_ONCHIP_ADC );							// Set certain bits.
	AD5421_Set( &spiData );									// Send data via SPI.
	spiData = AD5421_Get();									// Receive data via SPI.
	if ( spiData & cmdCheck )
	{
		// check the echo
		retValue = SUCCESS;
	}
	*ad5421_error = !retValue;

	return(retValue);
}

/**************************************************************************//**
 * @brief Write to the DAC register.
 *
 * @param dacValue - desired value to be written in register.
 *
 * @return None.
******************************************************************************/
void AD5421_SetDac( int dacValue )
{
	int spiData = 0;
	// Load data in DAC Register
	spiData = ( AD5421_CMDWRDAC << AD5421_CMDPOSITION );
	spiData += ( dacValue );
	AD5421_Set( &spiData );
}

/**************************************************************************//**
 * @brief Write to the Offset Adjust register.
 *
 * @param offsetValue - desired value to be written in register.
 *
 * @return None.
******************************************************************************/
void AD5421_SetOffset( int offsetValue )
{
	int spiData = 0;
	// Load data in OFFSET Register
	spiData = ( AD5421_CMDWROFFSET << AD5421_CMDPOSITION );
	spiData += ( offsetValue );
	AD5421_Set( &spiData );
}

/**************************************************************************//**
 * @brief Write to the Gain Adjust register
 *
 * @param gainValue - desired value to be written in register.
 *
 * @return None.
******************************************************************************/
void AD5421_SetGain( int gainValue )
{
	int spiData = 0;
	// Load data in GAIN Register
	spiData = ( AD5421_CMDWRGAIN << AD5421_CMDPOSITION );
	spiData += ( gainValue );
	AD5421_Set( &spiData );
}

/**************************************************************************//**
 * @brief read from the DAC register.
 *
 * @param  None.
 *
 * @return dacValue - value read from the register.
******************************************************************************/
int AD5421_GetDac( void )
{
	int spiData = 0;
	int dacValue = 0;
	// Read data from DAC Register
	spiData = ( AD5421_CMDRDDAC << AD5421_CMDPOSITION );
	AD5421_Set( &spiData );
	dacValue = AD5421_Get();

	return( dacValue );
}

/**************************************************************************//**
 * @brief Read from the Offset Adjust register.
 *
 * @param None.
 *
 * @return offsetValue - value calculated according to the datasheet formula.
******************************************************************************/
int AD5421_GetOffset( void )
{
	int spiData = 0;
	int offsetValue = 0;
	// Read data from OFFSET Register
	spiData = ( AD5421_CMDRDOFFSET << AD5421_CMDPOSITION );
	AD5421_Set( &spiData );
	offsetValue = AD5421_Get();
	// Calculate according to datasheet formula
	offsetValue = offsetValue - 32768;

	return( offsetValue );
}

/**************************************************************************//**
 * @brief Read from the Gain Adjust register
 *
 * @param None.
 *
 * @return gainValue - value calculated according to the datasheet formula
******************************************************************************/
int AD5421_GetGain( void )
{
	int spiData = 0;
	int gainValue = 0;
	// Read from GAIN Register
	spiData = ( AD5421_CMDRDGAIN << AD5421_CMDPOSITION );
	AD5421_Set( &spiData );
	gainValue = AD5421_Get();
	// Calculate according to datasheet formula
	gainValue = gainValue - 65535;

	return( gainValue );
}

/**************************************************************************//**
 * @brief Read from the Temp register.
 *
 * @param None.
 *
 * @return tempValue - temperature value in Celsius degrees.
******************************************************************************/
int AD5421_GetTemp( void )
{
	int spiData = 0;
	int tempValue = 0;
	// Set CONTROL Register to measure temperature
	spiData = ( AD5421_CMDWRCTRL << AD5421_CMDPOSITION );
	spiData += ( CTRL_AUTO_FAULT_RDBK | CTRL_SEL_ADC_INPUT | CTRL_ONCHIP_ADC );
	AD5421_Set( &spiData );
	// Initiate temperature measurement
	spiData = 0;
	spiData = ( AD5421_CMDMEASVTEMP << AD5421_CMDPOSITION );
	AD5421_Set( &spiData );
	// Read temperature from Fault Register
	spiData = 0;
	spiData = ( AD5421_CMDRDFAULT << AD5421_CMDPOSITION );
	AD5421_Set( &spiData );
	tempValue = ( unsigned int ) AD5421_Get();

	return( tempValue );
}

/**************************************************************************//**
 * @brief Read from the Vloop register.
 *
 * @param None.
 *
 * @return vlopValue - value of the voltage drop between Vloop and COM.
******************************************************************************/
float AD5421_GetVloop( void )
{
	int   spiData = 0;
	float vlopValue = 0;
	float divide_constant_vloop = 0.009765625f;	 // taken from datasheet formula
	// Set CONTROL Register to measure Vloop
	spiData = ( AD5421_CMDWRCTRL << AD5421_CMDPOSITION );
	spiData += ( CTRL_AUTO_FAULT_RDBK | CTRL_ONCHIP_ADC );
	AD5421_Set( &spiData );
	// Initiate VLOOP measurement
	spiData = 0;
	spiData = ( AD5421_CMDMEASVTEMP << AD5421_CMDPOSITION );
	AD5421_Set( &spiData );
	// Read VLOOP from Fault Register
	spiData = 0;
	spiData = ( AD5421_CMDRDFAULT << AD5421_CMDPOSITION );
	AD5421_Set( &spiData );
	// Calculate Vloop according to the datasheet formula
	vlopValue = ( float ) ( AD5421_Get() & 0x0FF );
	vlopValue = divide_constant_vloop * vlopValue;

	return( vlopValue );
}

/**************************************************************************//**
 * @brief Send data via SPI
 *
 * @param value: value to be sent via SPI
 *
 * @return SUCCESS.
******************************************************************************/
int AD5421_Set( int *value )
{
	txBuffer[0] = ( unsigned char ) ( ( *value & 0xff0000 ) >> 16 );	// Arrange the
	txBuffer[1] = ( unsigned char ) ( ( *value & 0x00ff00 ) >> 8 );		// data to be sent
	txBuffer[2] = ( unsigned char ) ( *value & 0x0000ff );				// in 8 bit packets
	alt_avalon_spi_command(AD5421_SPI, 0, 3, txBuffer, 0, rxBuffer, 0);

	return(SUCCESS);
}

/**************************************************************************//**
 * @brief Read data via SPI
 *
 * @param  None.
 *
 * @return returnVal - Retreived data
******************************************************************************/
int AD5421_Get( void )
{
	int returnVal = 0;
	txBuffer[0] = ( unsigned char ) 0;
	txBuffer[1] = ( unsigned char ) 0;
	txBuffer[2] = ( unsigned char ) 0;
	alt_avalon_spi_command(AD5421_SPI, 0, 0, txBuffer, 3, rxBuffer, 0);
	returnVal  = (int) ( rxBuffer[1] << 8 );
	returnVal += rxBuffer[2];

	return(returnVal);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return SUCCESS
******************************************************************************/
int AD5421_UpdateParameters(void)
{
	float 	divide_constant = 0.000244140625f; 							// taken from datasheet formula
	float 	divide_constant_optim = 0.0000000037252902984619140625f; 	// taken from datasheet formula
	int 	ad5421_temperatur;
	int 	ad5421_temp;
	int 	ad5421_fault;
	// Set DAC Value
	AD5421_SetDac( *ad5421_dacValue );
	// Read DAC Value
	*ad5421_dacValueGet = AD5421_GetDac();
	// Set OFFSET Value
	AD5421_SetOffset( *ad5421_offsetValue );
	// Read OFFSET Value
	*ad5421_offsetValueGet = AD5421_GetOffset();
	// Set GAIN Value
	AD5421_SetGain( *ad5421_gainValue );
	// Read GAIN Value
	*ad5421_gainValueGet = AD5421_GetGain();
	// Read Temperature and Fault Register
	ad5421_temp = AD5421_GetTemp();
	ad5421_temperatur = (ad5421_temp & 0x0FF);
	ad5421_fault = (ad5421_temp >> 8);
	// Set faults
	*faultVloop12 = ad5421_fault & (1<<0);
	*faultVloop6 = ad5421_fault & (1<<1);
	*faultTemp100 = ad5421_fault & (1<<2);
	*faultTemp140 = ad5421_fault & (1<<3);
	*faultIloopUnder = ad5421_fault & (1<<4);
	*faultIloopOver = ad5421_fault & (1<<5);
	*faultPec = ad5421_fault & (1<<6);
	*faultSpi = ad5421_fault & (1<<7);
	// Calculate temperature
	*calcTemp = (float) ( ad5421_temperatur - 128 );
	*calcTemp = 1.771 * *calcTemp;
	*calcTemp = 125 - *calcTemp;
	// Calculate loop current value without gain and offset
	*loopCurValue = divide_constant * *ad5421_dacValueGet + 4;
	// Calculate loop current value according to gain and offset
	*loopCurValueOptim = (( divide_constant_optim *
			( *ad5421_gainValueGet + 65535 ) * *ad5421_dacValueGet ));
	*loopCurValueOptim += ( divide_constant *
			( ( *ad5421_offsetValueGet + 32768 ) - 32768 ) );
	*loopCurValueOptim += 4;
	// Read VLoop Value
	*ad5421_vloop = AD5421_GetVloop();

	return(SUCCESS);
}
#endif
